/* system call prototype for PSP */

#ifndef _SYSCALL_H_INCLUDED
#define _SYSCALL_H_INCLUDED

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define DWORD u32
#define WORD  unsigned short
#define BYTE  u8
#define BOOL  int

typedef unsigned long	u32;
typedef unsigned short	u16;
typedef unsigned char	u8;
typedef unsigned char	bool;
typedef unsigned char	byte;

//typedef 		 int	BOOLEAN;
//typedef unsigned long	DWORD;
//typedef unsigned short	WORD;
//typedef unsigned char	BYTE;
//typedef 		 int	BOOL;

#define NULL 0

#define true 1
#define false 0
#define TRUE 1
#define FALSE 0

/******************************************************************************/
/* sceDisplay */

#define VRAM_ADDR	((char *)0x04000000)

#define SCREEN_WIDTH	480
#define SCREEN_HEIGHT	272

void sceDisplayWaitVblankStart();
void sceDisplaySetMode(long unknown, long width, long height);
void sceDisplaySetFrameBuf(char *topaddr,long linesize,long pixelsize,long);


/******************************************************************************/
/* sceCtrl */
/* Index for the two analog directions */ 
#define CTRL_ANALOG_X   0 
#define CTRL_ANALOG_Y   1 

/* Button bit masks */ 
#define CTRL_SQUARE		0x8000 
#define CTRL_TRIANGLE	0x1000 
#define CTRL_CIRCLE		0x2000 
#define CTRL_CROSS		0x4000 
#define CTRL_UP			0x0010 
#define CTRL_DOWN		0x0040 
#define CTRL_LEFT		0x0080 
#define CTRL_RIGHT		0x0020 
#define CTRL_START		0x0008 
#define CTRL_SELECT		0x0001 
#define CTRL_LTRIGGER	0x0100 
#define CTRL_RTRIGGER	0x0200 
#define	CTRL_HOLD	0x00020000

/* Returned control data */ 
typedef struct _ctrl_data 
{ 
   u32 frame; 
   u32 buttons; 
   u8  analog[4]; 
   u32 unused; 
} ctrl_data_t; 

void	sceCtrlInit				(int unknown); 
void	sceCtrlSetAnalogMode	(int on); 
void	sceCtrlRead				(ctrl_data_t* paddata, int unknown); 


/******************************************************************************/
/* IoFileMgrForUser */

#define MAX_PATH 512
#define MAX_NAME 256
#define MAX_ENTRY 1024

#define O_RDONLY    0x0001 
#define O_WRONLY    0x0002 
#define O_RDWR      0x0003 
#define O_NBLOCK    0x0010 
#define O_APPEND    0x0100 
#define O_CREAT     0x0200 
#define O_TRUNC     0x0400 
#define O_NOWAIT    0x8000 

enum { 
    TYPE_DIR=0x10, 
    TYPE_FILE=0x20 
}; 

struct dirent_tm {
	u16 unk[2]; //Ƀ[H
	u16 year;
	u16 mon;
	u16 mday;
	u16 hour;
	u16 min;
	u16 sec;
};

typedef struct _dirent_t { 
    u32 unk0; 
    u32 type; 
    u32 size; 
	struct dirent_tm ctime; //쐬
	struct dirent_tm atime; //ŏIANZX
	struct dirent_tm mtime; //ŏIXV
	u32 unk[7]; //Ƀ[H
    char name[0x108]; 
} dirent_t; 

int		sceIoOpen	(const char* file, int mode, int unknown); 
void	sceIoClose	(int fd); 
int		sceIoRead	(int fd, void *data, int size); 
int		sceIoWrite	(int fd, void *data, int size); 
int		sceIoLseek	(int fd, long long offset, int whence); 
int		sceIoRemove	(const char *file); 
int		sceIoMkdir	(const char *dir, int mode); 
int		sceIoRmdir	(const char *dir); 
int		sceIoRename	(const char *oldname, const char *newname); 
int		sceIoDopen	(const char *fn); 
int		sceIoDread	(int fd, dirent_t *de); 
void	sceIoDclose	(int fd); 

/******************************************************************************/
/* sceSuspendForUser */

/******************************************************************************/
/* ThreadManForUser */

typedef int (*psp_threadfunc_t)(int args, void *argp);

//int sceKernelCreateThread(const char *name, psp_threadfunc_t func, unsigned long priority, unsigned long stacksize, unsigned long attrib, void *unk);
int sceKernelStartThread(int hthread, int arg0, void *arg1);
void sceKernelExitThread(int ret);
int sceKernelWaitThreadEnd(int hthread, void *unk);
int sceKernelDeleteThread(int hthread);

void	sceKernelPollCallbacks(void);
//int		sceKernelCreateCallback(const char *name, void (*func)());

/******************************************************************************/
/* LoadExecForUser */

void sceKernelExitGame();
void sceKernelRegisterExitCallback(int cbid);

/******************************************************************************/
/* scePower */

#define POWER_CB_POWER		0x80000000 
#define POWER_CB_HOLDON		0x40000000 
#define POWER_CB_STANDBY	0x00080000 
#define POWER_CB_RESCOMP	0x00040000 
#define POWER_CB_RESUME		0x00020000 
#define POWER_CB_SUSPEND	0x00010000 
#define POWER_CB_EXT		0x00001000 
#define POWER_CB_BATLOW		0x00000100 
#define POWER_CB_BATTERY	0x00000080 
#define POWER_CB_BATTPOWER	0x0000007F 

void scePowerRegisterCallback(int zero, int cbid);
void scePowerSetClockFrequency(int,int,int);

/******************************************************************************/
/* UtilsForUser */

struct timeval {
	long tv_sec;
	long tv_usec;
};

struct sce_timezone {
  int tz_minuteswest;
  int tz_dsttime;
};


unsigned long sceKernelLibcClock(void);
unsigned long sceKernelLibcTime(unsigned long *);

/******************************************************************************/
/* sceAudio */
void sceAudioOutputBlocking();//
void sceAudioOutputPanned();//
long sceAudioOutputPannedBlocking(long, long, long, void *);//
long sceAudioChReserve(long, long samplecount, long);//init buffer? returns handle, minus if error
void sceAudioChRelease(long handle);//free buffer?
void sceAudioGetChannelRestLen();//
long sceAudioSetChannelDataLen(long, long);//
void sceAudioChangeChannelConfig();//
void sceAudioChangeChannelVolume();//

/******************************************************************************/
/* SysMemUserForUser */

int sceKernelAllocPartitionMemory(void* buf,int size); 
void sceKernelFreePartitionMemory(void* buf);






#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif // _SYSCALL_H_INCLUDED
